<?php
namespace Controller;

use Service\DeleteCascadeService;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO;
use Service\Traits\TResponse;
use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Util\Utilities;
use Service\TemplateJson;
use \Service\Dashboard;
use Service\PeriodObservation;
use Service\Indicator;
use \Service\ChartView\ChartViewFactory as ChartViewFactory;

class Widget extends Crud {

    use TResponse;

    protected $dataSourceName = "widget";

    protected $organizationMongoManager;
    /** @var \Service\Widget $widgetService */
    private $widgetService;
    /** @var \Service\Dashboard $dashboardService */
    protected $dashboardService;
    /** @var \Service\PeriodObservationService $periodObservationService */
    protected $periodObservationService;

    public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, \Service\Widget $widgetService, Dashboard $dashboardService, PeriodObservation $periodObservationService) {
        $this->widgetService = $widgetService;
        $this->organizationMongoManager = $organizationMongoManager;
        $this->dashboardService = $dashboardService;
        $this->periodObservationService = $periodObservationService;
        parent::__construct($organizationMongoManager, $deleteCascadeService);
    }

    public function buildWidgetByViewIdNext(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            if (!empty($row['viewId'])) {
                $viewId      = $row['viewId'];
                $facts       = isset($row['facts'])                         ? $row['facts']             : array();
                $orderBy     = isset($row['orderBy'])                       ? $row['orderBy']           : array();
                $period      = isset($row['period'])                        ? $row['period']            : array();
                $structureId = isset($row['structureId'])                   ? $row['structureId']       : null;
                $structureName = isset($row['structureName'])               ? $row['structureName']     : null;
                $dimension   = isset($row['dimension'])                     ? $row['dimension']         : null;
                $limitRows   = isset($row['limitRows'])                     ? $row['limitRows']         : null;
                $drillDownFilters = isset($row['drillDownFilters'])         ? $row['drillDownFilters']  : null;
                $valuePrecision = isset($row['valuePrecision'])             ? $row['valuePrecision']    : null;
                $serieColors = isset($row['serieColors']) && $row['serieColors']  ? $row['serieColors'] : null;
                $dimensionX     = isset($row['dimensionX'])      ? $row['dimensionX']       : null;
                $dimensionY     = isset($row['dimensionY'])      ? $row['dimensionY']       : array();
                $isViewPreview = empty($row["isViewPreview"]) ? false : true;
                $isChartView = empty($row["isChartView"]) ? false : true;
                $dashboardId = !empty($row["dashboardId"]) ? $row['dashboardId']  : null;
                $label = !empty($row["label"]) ? $row["label"]  : null;

                $widget = $this->widgetService->buildWidgetToRender($viewId, $structureId,
                    $period, $dimension, $facts, $orderBy, $limitRows, $drillDownFilters,
                    $valuePrecision, null, null, $serieColors, null, $dimensionX, $dimensionY, $structureName, $isViewPreview);
                if(!empty($label)){
                    $widget["name"] = $label;
                }
                if(isset($widget['legendClosed'])){
                    $chartLegendClosedValue  = $widget['legendClosed'];
                    unset($widget['legendClosed']);
                    $widget['chartLegendClosed']  = $chartLegendClosedValue;
                }
                $this->dashboardService->fixBugGauge($widget);
                if($widget["chartSeries"] && is_array($widget["chartSeries"])){
                    foreach($widget["chartSeries"] as &$chart){
                        $this->dashboardService->fixBugGauge($chart);
                    }
                }
                $widget["isChartView"] = $isChartView;
                $this->widgetService->getWidgetLastImportDate($widget);
                $widget = ChartViewFactory::factory($widget, $dashboardId)->convertToNext();
                $response->addDataSet(new DataSet('widget', array($widget)));
//                $observationsPeriod = empty($widget["parameters"]["period"]) ? null : $widget["parameters"]["period"];
//                $observations = $this->periodObservationService->getObservationsByDateAndIndicatorAndDataSource(
//                    $observationsPeriod,$widget["indicator"],$widget["dataSourceId"]);
//                $response->addDataSet(new DataSet('periodObservationNotification', $observations));
            } else {
                $response->setError(new DTO\Response\Error("Invalid parameters. Missing the indicator view id.", 500));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findNextByIndicator(DTO\Request $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();
            $dataSet = $this->widgetService->findNext($params);
            $dataSet = Utilities::parseDataSourceZeedhiNext($dataSet, $this->count($params));
            $dataSet["pagination"]["page"] = isset($params["page"]) ? (int)$params["page"] : 1;
            $dataSet["pagination"]["limit"] = isset($params["limit"]) ? (int)$params["limit"] : 9999;
            $this->sendResponse($dataSet, 200);

        } catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function count($params) {
        return $this->mongoManager->countNext($this->dataSourceName, $params);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        parent::find($request, $response);
        $dataSet = $response->getDataSets()[0]->getRows();
        $dataSet = $this->setPeriod($dataSet);
        $response->addDataSet(new DataSet("widget", $dataSet));
    }

    public function findWithIndicatorNamesNext(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        Utilities::convertDynamicFilterToArray($params);
        if (array_key_exists("indicator", $params)) {
            $params["dynamic_filter"]["indicator"] = [["relation"=>  "AND", "operation" => "EQUALS", "value"=> $params["indicator"]]];
        }
        $dataSet = $this->mongoManager->findNext($this->dataSourceName, $params, false);
        $total = $this->mongoManager->countNext($this->dataSourceName, $params, false);
        $dataSet = $this->setPeriod($dataSet);
        $dataSet = $this->widgetService->findWithIndicatorNames($dataSet);
        $dataSet = Utilities::parseDataSourceZeedhiNext($dataSet, $total);
        $dataSet['pagination']['page'] = array_key_exists('page', $params) ? intval($params['page']) : 1;
        $dataSet['pagination']['limit'] = array_key_exists('limit', $params) ? intval($params['limit']) : 99999;
        $this->sendResponse($dataSet, 200);
    }

    public function getDefaultOrgViews(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $views = $this->widgetService->getDefaultOrgViews($filterCriteria);
            $response->addDataSet(new DataSet($this->dataSourceName, $views));

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

	public function save(DTO\Request $request, DTO\Response $response) {
        $widget = $request->getDataSet()->getRows()[0];
        if (isset($widget['viewTypeId'])) {
            $widget = $this->validateProperties($widget);
        }
        if(Utilities::getOrganizationIdForMongo()== '5'){
            $widget["mockPeriod"] = empty($widget["period"]) ? null : $this->widgetService->setPeriodFormatMock($widget["period"],$widget['indicator']);
        }
		if(isset($widget['_id'])){
		    $this->updateWidgetDrillDownName($widget);
		    $this->updateDashboardWidgetName($widget);
        }
        if(!empty($widget["drillDownViewFieldWithoutIndicator"])){
            $widget["drillDownViewField"] = $widget["drillDownViewFieldWithoutIndicator"];
        }

        $dataBefore = null;
        if(!empty($widget['_id'])){
            $dataBefore = $this->widgetService->findById($widget['_id']);
        }
        parent::save($request, $response);
        $id = $response->getDataSets()[0]->getRows()[0]['_id'];
        $this->widgetService->setLog($dataBefore,$id);
    }

    public function delete(DTO\Request $request, DTO\Response $response)
    {
        $widgets = $request->getDataSet()->getRows();
        $dataBefore = array();
        foreach ( $widgets as $key => $widget) {
            if(!empty($widget['_id'])){
                $dataBefore[$key] = $this->widgetService->findById($widget['_id']);
            }
        }

        parent::delete($request, $response);

        foreach ( $widgets as $key => $widget) {
            if (!empty($widget['_id'])) {
                $this->widgetService->setLog($dataBefore[$key], $widget['_id']);
            }
        }
    }

    protected function validateProperties (\Zeedhi\Framework\DTO\Row $widget){
        $widgetViewType = TemplateJson::find(TemplateJson::getViewType(), $widget['viewTypeId']);
        $widgetViewProperties = TemplateJson::findViewPropertyByIds($widgetViewType['properties']);
        if(isset($widgetViewType['extraProperties'])){
            $widgetViewExtraProperties = TemplateJson::findViewPropertyByIds($widgetViewType['extraProperties']);
            $widgetViewProperties = array_merge($widgetViewProperties,$widgetViewExtraProperties);
        }
        $viewProperties = TemplateJson::getViewProperty();
        $widgetProperties = array_keys(get_object_vars($widget));
        $viewPropertiesNames = [];
        $viewPropertiesOutDataNames = [];
        $widgetViewPropertiesNames = ['serieType'];
        $widgetViewPropertiesOutDataNames = [];

        foreach ($widgetViewProperties as $widgetViewProperty){
            if (isset($widgetViewProperty['name'])){
                $widgetViewPropertiesNames[] = $widgetViewProperty['name'];
            }
            if (isset($widgetViewProperty['outData']['name'])){
                $widgetViewPropertiesNames[] = $widgetViewProperty['outData']['name'];
            }
        }

        foreach ($viewProperties as $viewProperty){
            if (isset($viewProperty['name'])){
                $viewPropertiesNames[] = $viewProperty['name'];
            }
            if (isset($viewProperty['outData']['name'])){
                $viewPropertiesNames[] = $viewProperty['outData']['name'];
            }
        }
        if((isset($widget['chartType']) && $widget['chartType'] === 'bar')){
            $widgetViewPropertiesNames[] = 'chartInvertedAxes';
        }

        foreach ($widgetProperties as $property){
            if (!(in_array($property, $widgetViewPropertiesNames) || in_array($property, $widgetViewPropertiesOutDataNames)) &&
                (in_array($property, $viewPropertiesNames) || in_array($property, $viewPropertiesOutDataNames))){
                $widget->offsetUnset($property);
            }
        }
        return $widget;
    }

	protected function updateWidgetDrillDownName($widget){
	    $dataSourceName = $this->dataSourceName;
	    $widgetFilterCriteria = new FilterCriteria($dataSourceName);
	    $widgetFilterCriteria->addCondition('drillDownView', $widget['_id']);
	    $widgetsToUpdate = $this->organizationMongoManager->findBy($widgetFilterCriteria)->getRows();

	    foreach ($widgetsToUpdate as &$widgetToUpdate) {
	        $widgetToUpdate['drillDownViewField'] = $widget['label'];
	    }

	    $dataSet = new DataSet($dataSourceName, $widgetsToUpdate);
	    $persistedRows = $this->organizationMongoManager->persist($dataSet);
	}

	protected function updateDashboardWidgetName($widget){
	    $dataSourceName = 'view';
	    $dashboardFilterCriteria = new FilterCriteria($dataSourceName);
	    $dashboardFilterCriteria->addCondition('type', 'D');
	    $dashboardsToUpdate = $this->organizationMongoManager->findBy($dashboardFilterCriteria)->getRows();

	    foreach ($dashboardsToUpdate as &$dashboard) {
	        if(isset($dashboard['indicators'])){
	            foreach ($dashboard['indicators'] as &$dashboardIndicator) {
	                if($dashboardIndicator['widget'] == $widget['_id']){
	                    $dashboardIndicator['widget_label'] = $widget['label'];
	                }
	            }
	        }
	    }

	    $dataSet = new DataSet($dataSourceName, $dashboardsToUpdate);
	    $persistedRows = $this->organizationMongoManager->persist($dataSet);
	}

	public function getColumnsWidget(DTO\Request $request, DTO\Response $response){
        $params = $request->getParameters();
        if (isset($params["widget"])){
            $widget = $params["widget"];
            Utilities::convertDynamicFilterToArray($params);
            // $params["dynamic_filter"]["_id"] = [["relation"=>  "AND", "operation" => "EQUALS", "value"=> $params["widget"]]];
            $widgetsColumns = $this->organizationMongoManager->findNext($this->dataSourceName, $params, false);
            if(isset($widgetsColumns[0]["orderBy"])){
                $orderBy = $widgetsColumns[0]["orderBy"];
            }else{
                $orderBy = null;
            }
            if(isset($widgetsColumns[0]["columns"])){
                $widgetsColumns = $widgetsColumns[0]["columns"];
            }
            $widgetsColumns = $this->widgetService->columnOrderBy($widgetsColumns,$orderBy);
            
        } else {
            $widgetsColumns = array();
        }
        $data = Utilities::parseDataSourceZeedhiNext($widgetsColumns, count($widgetsColumns));
        $data["pagination"]["page"] = isset($params["page"]) ? (int)$params["page"] : 1;
        $data["pagination"]["limit"] = isset($params["limit"]) ? (int)$params["limit"] : 9999;
        $this->sendResponse($data, 200);
    }

    public function dimensionFind(DTO\Request $request, DTO\Response $response){
        try{
            $dataSet = $this->widgetService->reserchForEach($request,\Service\Widget::IS_DIMENSION);
            $data = Utilities::parseDataSourceZeedhiNext($dataSet, count($dataSet));
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function factFind(DTO\Request $request, DTO\Response $response){
        try{
            $dataSet = $this->widgetService->reserchForEach($request,\Service\Widget::IS_FACT);
            $data = Utilities::parseDataSourceZeedhiNext($dataSet, count($dataSet));
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    protected function setPeriod(array $dataSet)
    {
        return Utilities::setPeriod($dataSet);
    }
}
