import { Loader, Metadata } from '@zeedhi/core';
import { Modal, ModalService, DialogService, LoadingService, Container } from '@zeedhi/common';
import { DataSourceService } from '@/Services/DataSourceService';
import { CustomAlertService } from '@/Services/CustomAlertService';
import { DataSourceController } from './DataSourceController';

export class DSStructureController {
    private mainController: DataSourceController = Loader.getInstance("DataSourceController");
    public service = new DataSourceService();

    private get datasourceData() {
        return this.mainController.datasourceData;
    }

    private structureModal!: Modal;
    public structureData: any = {};
    public async openStructuresModal({ row }: any) {
        this.structureModal?.destroy();
        this.structureModal = await ModalService.createFromJson('dataSource/RelateStructures/RelateStructuresModal', true);
        this.structureData = row || {};
        this.structureModal.show();
    }
    public closeStructuresModal() {
        this.structureModal.hide();
    }
    public structureModalMounted({ component }: any) {
        const listEl = component;
        const relationsValues = this.datasourceData.relatedStructuresType.find((el: any) => el.structureType === this.structureData._id)?.relations || [];
        const relations = this.structureData.fieldKey.map((key: string) => ({ key, value: relationsValues.filter((relation: any) => relation.structureColumn === key).map((el: any) => el.dataSourceColumn) }));
        listEl.children = relations.map(this.generateRelationListItem);
    }
    public generateRelationListItem = (relation: any, id: number) => ({
        "name": "relation_" + relation.key,
        "component": "ZdRow",
        "dense": false,
        "cssClass": "zd-justify-space-between zd-align-center zd-py-4",
        "children": [
            {
                "name": "relationName_" + id,
                "component": "ZdText",
                "text": relation.key,
                "readonly": true,
                "showLabel": false,
                "showHelper": false,
                "cssClass": "zd-col-4",
                "disabled": true
            },
            {
                "name": "relationSelect_" + id,
                "component": "ZdSelectMultiple",
                "value": relation.value,
                "showHelper": false,
                "showLabel": false,
                "disabledItems": "{{DSStructureController.structureRelationSelectDisabled}}",
                "cssClass": "zd-col-7",
                "dataText": "label",
                "dataValue": "name",
                "datasource": {
                    "type": "memory",
                    "uniqueKey": "name",
                    "data": this.datasourceData.relations.map(({name, label}: any) => ({name, label})) || []
                },
                "events": {
                    "focus": "{{DSStructureController.structureRelationSelectFocus}}"
                }
            }
        ]
    });
    public currentStructureRelationSelect: string = "";
    public structureRelationSelectFocus({ component }: any) {
        this.currentStructureRelationSelect = component.parent.name.split(/_(.+)/)[1];
    }
    public get structureRelationSelectDisabled() {
        const list = Metadata.getInstance<Container>("structureDetailList");
        const values = list.childrenInstances?.map((el: any) => ({name: el.name.split(/_(.+)/)[1], value: el.childrenInstances[1]?.value})) || [];
        return values.filter((el: any) => el.name !== this.currentStructureRelationSelect).map((el: any) => el.value).flat();
    }
    public async saveStructuresModal() {
        const list = Metadata.getInstance<Container>("structureDetailList");
        const values = list.childrenInstances?.map((el: any) => ({name: el.name.split(/_(.+)/)[1], value: el.childrenInstances[1]?.value})) || [];
        let dataHolder = this.datasourceData.relatedStructuresType.find((el: any) => el.structureType === this.structureData._id);
        if (!dataHolder) {
            dataHolder = { structureType: this.structureData._id, relations: [] };
            this.datasourceData.relatedStructuresType.push(dataHolder);
        }
        dataHolder.relations = values.map((el: any) => el.value.map((value: any) => ({structureColumn: el.name, dataSourceColumn: value})).flat()).flat();
        try {
            const res = await this.service.saveDataSource(this.datasourceData, this.mainController.user.userId);
            if (res.data.messages?.length > 0) {
                const message = res.data.messages[0].message;
                DialogService.show({
                  name: 'warning-dialog',
                  type: 'error',
                  title: ' ',
                  text: message
                });
            } else {
                CustomAlertService.saveSuccessAlert();
            }
            this.closeStructuresModal();
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }
}
